@@ -9,10 +9,24 @@ pyarrow.RecordBatchReader
 import pandas
 
 pandas.DataFrame
+pandas.Categorical
+pandas.CategoricalDtype
+pandas.Series
 pandas.NaT
 pandas.NA
 pandas.isnull
 pandas.ArrowDtype
+pandas.BooleanDtype
+pandas.UInt8Dtype
+pandas.UInt16Dtype
+pandas.UInt32Dtype
+pandas.UInt64Dtype
+pandas.Int8Dtype
+pandas.Int16Dtype
+pandas.Int32Dtype
+pandas.Int64Dtype
+pandas.Float32Dtype
+pandas.Float64Dtype
 
 import datetime
 
@@ -46,6 +60,7 @@ import numpy
 
 numpy.core.multiarray
 numpy.ma.masked
+numpy.ma.masked_array
 numpy.ndarray
 numpy.datetime64
 numpy.generic
