@@ -71,6 +71,7 @@ DefaultConnectionHolder DuckDBPyConnection::default_connection;
 DBInstanceCache instance_cache;                                                        // NOLINT: allow global
 shared_ptr<PythonImportCache> DuckDBPyConnection::import_cache = nullptr;              // NOLINT: allow global
 PythonEnvironmentType DuckDBPyConnection::environment = PythonEnvironmentType::NORMAL; // NOLINT: allow global
+std::string DuckDBPyConnection::formatted_python_version = "";
 
 DuckDBPyConnection::~DuckDBPyConnection() {
 	try {
@@ -83,6 +84,13 @@ DuckDBPyConnection::~DuckDBPyConnection() {
 }
 
 void DuckDBPyConnection::DetectEnvironment() {
+	// Get the formatted Python version
+	py::module_ sys = py::module_::import("sys");
+	py::object version_info = sys.attr("version_info");
+	int major = py::cast<int>(version_info.attr("major"));
+	int minor = py::cast<int>(version_info.attr("minor"));
+	DuckDBPyConnection::formatted_python_version = std::to_string(major) + "." + std::to_string(minor);
+
 	// If __main__ does not have a __file__ attribute, we are in interactive mode
 	auto main_module = py::module_::import("__main__");
 	if (py::hasattr(main_module, "__file__")) {
@@ -120,6 +128,10 @@ bool DuckDBPyConnection::IsJupyter() {
 	return DuckDBPyConnection::environment == PythonEnvironmentType::JUPYTER;
 }
 
+std::string DuckDBPyConnection::FormattedPythonVersion() {
+	return DuckDBPyConnection::formatted_python_version;
+}
+
 // NOTE: this function is generated by tools/pythonpkg/scripts/generate_connection_methods.py.
 // Do not edit this function manually, your changes will be overwritten!
 
@@ -2111,9 +2123,9 @@ shared_ptr<DuckDBPyConnection> DuckDBPyConnection::Connect(const py::object &dat
 	    "If set, restores the old behavior of scanning all preceding frames to locate the referenced variable.",
 	    LogicalType::BOOLEAN, Value::BOOLEAN(false));
 	if (!DuckDBPyConnection::IsJupyter()) {
-		config_dict["duckdb_api"] = Value("python");
+		config_dict["duckdb_api"] = Value("python/" + DuckDBPyConnection::FormattedPythonVersion());
 	} else {
-		config_dict["duckdb_api"] = Value("python jupyter");
+		config_dict["duckdb_api"] = Value("python/" + DuckDBPyConnection::FormattedPythonVersion() + " jupyter");
 	}
 	config.SetOptionsByName(config_dict);
 
